import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import quad
import scipy
import pandas as pd

ℏ     =  1.054571817e-34 #J⋅s
Me    =  9.1093837E-31 # kg, mass of electron
eV    =  1.602176634e-19 #J, electronvolt
k     =  1.380649e-23 #J/K, Boltzman

Mg    =  0.043*Me #kg, mass at gamma point
Ml    =  0.4412*Me #kg, DOS mass at L point
Mx    =  0.7576*Me #kg, DOS mass at X point


alpha =  1.198/eV #(J⋅s)^-1
dEl   =  0.511*eV #threshold energy, Eg-El
dEx   =  dEl+0.063*eV #threshold energy, Ex-Eg
n0    =  8.6545e18 #cm-3, initial carrier concentration


#electron density at Gamma point
def Ng(E,T,Ef):
    return 1/(2*np.pi**2)*(2*Mg/ℏ**2)**1.5*(E*(1+E*alpha))**0.5*(1+2*E*alpha) / (np.exp((E-Ef)/(k*T)) + 1)
 
#electron density at l point    
def Nl(E,T,Ef):
        return 1/(2*np.pi**2)*(2*Ml/ℏ**2)**1.5*(E-dEl)**0.5 / (np.exp((E-Ef)/(k*T)) + 1)

#electron density at l point    
def Nx(E,T,Ef):
        return 1/(2*np.pi**2)*(2*Mx/ℏ**2)**1.5*(E-dEx)**0.5 / (np.exp((E-Ef)/(k*T)) + 1)

# total electron distribution  =  total electron density* Fermi distribution        
def Ne(E,T,Ef):
    return Ng(E,T,Ef) + Nl(E,T,Ef) + Nx(E,T,Ef)

#fucntion of difference calculated electron density and n0
def n(Ef,T): 
    return quad(lambda E: Ng(E,T,Ef), 0, 10*eV)[0]/10**6 + quad(lambda E: Nl(E,T,Ef), dEl, 10*eV)[0]/10**6 +\
           quad(lambda E: Nx(E,T,Ef), dEx, 10*eV)[0]/10**6 - n0 #cm-3

# calculation of Ef for every temperature
T    = np.linspace(300, 2500, 100)
Ef   = np.zeros(len(T))
for i in range (0, len(T),1):
    Ef[i] = scipy.optimize.fsolve(n, 0.25*eV, args=T[i])

def m(E):
    return Mg*(1+2*E*alpha)

    
meff = np.zeros(len(T))
Nl0 = np.zeros(len(T)) 
Ng0 = np.zeros(len(T)) 
Nx0 = np.zeros(len(T)) 
fg  = np.zeros(len(T)) 
fl  = np.zeros(len(T)) 
fx  = np.zeros(len(T)) 

for i in range (0, len(T),1):
    Nl0[i] = quad(lambda E: Nl(E,T[i],Ef[i]), dEl, 10*eV)[0]/10**6 #electron density at L point
    Ng0[i] = quad(lambda E: Ng(E,T[i],Ef[i]), 0, 10*eV)[0]/10**6 #electron density at Gamma point 
    Nx0[i] = quad(lambda E: Nx(E,T[i],Ef[i]), dEx, 10*eV)[0]/10**6 #electron density at X point
    fx [i] = Nx0[i]/n0           #fraction of X electrones
    fg [i] = Ng0[i]/n0            #fraction of Gamma electrones
    fl [i] = Nl0[i]/n0            #fraction of L electrones
    meff[i] = 1/( fg[i]*quad(lambda E: Ng(E,T[i],Ef[i])/m(E), 0, 10*eV)[0] / quad(lambda E: Ng(E,T[i],Ef[i]), 0, 10*eV)[0] + fl[i]/Ml + fx[i]/Mx)/Me
    

 
fig = plt.figure(0)
plt.plot(T, meff)
plt.xlabel("Tempearure[K]", fontsize=16)
plt.ylabel("meff/m0", fontsize=16)

fig = plt.figure(1)
plt.plot(T, fl)
plt.xlabel("Tempearure[K]", fontsize=16)
plt.ylabel("f, fraction of L electrons", fontsize=16)


fig = plt.figure(2)
dE = np.linspace(0, 2*eV, 2000)
i=0
plt.title('Charge carrier distribution, T='+np.str(np.round(T[i],1))+', m*='+np.str(np.round(meff[i],3)), fontsize=10)
plt.plot(Ng(dE,T[i],Ef[i])/10**6, dE/eV, label='Gamma')
plt.plot(Nl(dE,T[i],Ef[i])/10**6, dE/eV, label="L")
plt.plot(Nx(dE,T[i],Ef[i])/10**6, dE/eV, label="X")
plt.xlabel("specific carrier concentration, cm-3", fontsize=10)
plt.ylabel("Energy, eV", fontsize=10)
plt.legend(fontsize=15)


fig = plt.figure(3)
i=99
plt.title('Charge carrier distribution, T='+np.str(np.round(T[i],1))+', m*='+np.str(np.round(meff[i],3)), fontsize=10)
plt.plot(Ng(dE,T[i],Ef[i])/10**6, dE/eV, label='Gamma')
plt.plot(Nl(dE,T[i],Ef[i])/10**6, dE/eV, label="L")
plt.plot(Nx(dE,T[i],Ef[i])/10**6, dE/eV, label="X")
plt.xlabel("specific carrier concentration, cm-3", fontsize=10)
plt.ylabel("Energy, eV", fontsize=10)
plt.legend(fontsize=15)

fig = plt.figure(4)
plt.plot(T, fx)
plt.xlabel("Tempearure[K]", fontsize=16)
plt.ylabel("f, fraction of X electrons", fontsize=16)
